#ifndef FUNCTIONS_H
#define FUNCTIONS_H

// WiFi ////////////////////////////////////////////////////////////////////////

void WiFi_init();

int WiFi_strength();

// DS18B20 /////////////////////////////////////////////////////////////////////
struct DS18B20_Readings
{
    char *Name[10];
    float Temperature[10];
};

int DS18B20_init();
void DS18B20_main();
float DS18B20_getReading(int deviceNumber);
// struct DS18B20_Readings DS18B20_getReading(int deviceNumber);
void DS18B20_SendData(struct DS18B20_Readings Data);
void DS18B20_debug(struct DS18B20_Readings Datum);
// void DS18B20_debug();

// BME280 //////////////////////////////////////////////////////////////////////
struct BME280_Readings
{
    float Temperature;
    float Humidity;
    float AbsPressure;
    float SeaPressure;
};

int BME280_init();
void BME280_main();
struct BME280_Readings BME280_getReadings(int deviceNumber);
void BME280_SendData(struct BME280_Readings);
void BME280_debug(struct BME280_Readings);

// Si7021 //////////////////////////////////////////////////////////////////////
struct Si7021_Readings
{
    float Temperature;
    float Humidity;
};

int Si7021_init();
void Si7021_main();
struct Si7021_Readings Si7021_getReadings(int deviceNumber);
void Si7021_SendData(struct Si7021_Readings);
void Si7021_debug(struct Si7021_Readings);

// Hih6130 /////////////////////////////////////////////////////////////////////
struct hih6130_Readings
{
    byte Status;

    float Temperature;
    float Temperature_F;
    float Humidity;
    float HeatIndex;
    float HeatIndex_F;
};

int HiH6130_init();
void Hih6130_main();
struct hih6130_Readings hih6130_getReadings(int deviceNumber);
void hih6130_SendData(struct hih6130_Readings Data);
void hih6130_debug(struct hih6130_Readings Datum);

// MQTT ////////////////////////////////////////////////////////////////////////

void MQTT_init();

void MQTT_callback(char *topic, byte *payload, int length);
void MQTT_reconnect();
void MQTT_beacon();

void DOTheMQTT();

void MQTT_SendJSON(char *JSONstream);

// SSD1306 OLED ////////////////////////////////////////////////////////////////

void SSD1306_init();
bool SSD1306_Exists();
void SSD1306_title(const char *Title);
void SSD1306_Static(const char *Line, int Row);
void SSD1306_Display_DS18B20_Temps(struct DS18B20_Readings);
void SSD1306_Display_BME280_Temps(struct BME280_Readings);
void SSD1306_Display_Si7021_Temps(struct Si7021_Readings Datum);
void SSD1306_Display_HiH6130_Temps(struct hih6130_Readings Datum);

void SSD1306_Indicate_Action();

void WalkingDude();
void WavingDude();
void beerStickDude(int LEFT, int TOP);

/*
 * Possible future features
 */
void SSD1306_off();
void SSD1306_on();
void SSD1306_Dim();
void SSD1306_Bright();

// Supplimental Stuff //////////////////////////////////////////////////////////

void blinkLED(int speed); // Just a little function to blink the LED

// void PickOne(); // Can't remember off-hand why I created this... hhhmmm...

bool I2C_Test(int SDA, int SCL); // Sometimes I2C acts dumb on reset of the MCU
int I2C_ClearBus(int SDA, int SCL);

// Debug ///////////////////////////////////////////////////////////////////////
void debug_TitleScreen();                   // Title block for the project
void debug_ReadyScreen();                   // Indicate setup is finished
void debug_Separator();                     // Make a line
void debug_SectionTitle(const char* Title); // Text at the left
void debug_LineOut(const char* Line);       // Text 3 characters inset
void debug_Action(const char* Line);        // Text centred
void debug_Trouble(const char* Line);       // 
void debug_Success(const char* Line);       // 
void debug_ProgressBar0();
void debug_ProgressBar1();
void debug_ProgressBar2(int dotcount);

#endif // FUNCTIONS_H
