#include "libraries.h"
#include "functions.h"

#include "MQTT.h"

WiFiClient espClient;
PubSubClient MQTT_client(espClient);

void DOTheMQTT()
{
    if (!MQTT_client.connected())
    {
        MQTT_reconnect();
    }
    MQTT_client.loop();
    // MQTT_beacon();
}

////////////////////////////////////////////////////////////////
// MQTT Functions
////////////////////////////////////////////////////////////////

void MQTT_init()
{
    MQTT_client.setServer(MQTT_broker, 1883);
    MQTT_client.setCallback(MQTT_callback);

    //  Build the topic names
    strcpy(MQTT_inTopic, "cmnd/"); //  in - Commands
    strcat(MQTT_inTopic, MQTT_ClientName);
    strcat(MQTT_inTopic, "/#");
    strcpy(MQTT_teleTopic, "tele/"); // out - Telemetry
    strcat(MQTT_teleTopic, MQTT_ClientName);
    strcpy(MQTT_statTopic, "stat/"); // out - Status
    strcat(MQTT_statTopic, MQTT_ClientName);
    strcpy(MQTT_outTopic, "noti/"); // out - Notifications
    strcat(MQTT_outTopic, MQTT_ClientName);

    MQTT_reconnect();
}

void MQTT_callback(char *topic, byte payload[100], int length)
{
    char debugTEXT[46];

    char MQTT_msg_in[MQTT_BUFFER_SIZE];
    char *MQTT_command = strrchr(topic, '/');
    char CNasT[MQTT_BUFFER_SIZE];
    strcpy(CNasT, "/");
    strcat(CNasT, MQTT_ClientName); // "ClientName as Topic"

#ifdef DEBUG4
    debug_SectionTitle("Message arrived");
    sprintf(debugTEXT, "Topic: %30s", MQTT_command);
    debug_LineOut(debugTEXT);
#endif

    // if (length < MQTT_BUFFER_SIZE)
    // if (length < 63)
    // Messages 63 characters long or bigger make BOOM
    // with a "Soft WDT reset"
    // MQTT_BUFFER_SIZE is 100
    // I R cornfoozed...
    if (length < 59) // & now 59 is bad...  :|
    {

        MQTT_msg_in[0] = '\0'; // start with an empty string!
        for (int i = 0; i < length; i++)
        {
            MQTT_msg_in[i] = (char)payload[i];
            MQTT_msg_in[i + 1] = '\0';
        }

#ifdef DEBUG4
        sprintf(debugTEXT, "Message: %28s", MQTT_msg_in);
        debug_LineOut(debugTEXT);
        sprintf(debugTEXT, "Message Size: %d", length);
        debug_LineOut(debugTEXT);
#endif

        /////////////////////////////////////////////////////
        // Message handling goes here...
        /////////////////////////////////////////////////////

        if (strcmp(MQTT_command, CNasT) == 0) // MQTT_ClientName as Topic
        {
#ifdef DEBUG4
            debug_Trouble("Missing topic...");
#endif
            // Missing topic
        }
        else if (strcmp(MQTT_command, "/Sumpin") == 0)
        {
            if (strcmp(MQTT_msg_in, "") == 0)
            {
#ifdef DEBUG4
                debug_Trouble("No actual message...");
#endif
                // Might be a good way to request status...
            }
            else if ((strcmp(MQTT_msg_in, "0") == 0))
                debug_Action("Sumpin 0 Selected");
            else if ((strcmp(MQTT_msg_in, "1") == 0))
                debug_Action("Sumpin 1 Selected");
            else if ((strcmp(MQTT_msg_in, "2") == 0))
                debug_Action("Sumpin 2 Selected");
            else if ((strcmp(MQTT_msg_in, "3") == 0))
                debug_Action("Sumpin 3 Selected");
            else
                debug_Trouble("Say What?");
        }
        else if (strcmp(MQTT_command, "/Sensor") == 0) // Sensor selection
        {
            if (strcmp(MQTT_msg_in, "") == 0)
            {
#ifdef DEBUG4
                debug_Trouble("No actual message...");
#endif
                // Might be a good way to request status...
            }
            else if ((strcmp(MQTT_msg_in, "0") == 0))
                debug_Action("Sensor 0 Selected");
            else if ((strcmp(MQTT_msg_in, "1") == 0))
                debug_Action("Sensor 1 Selected");
            else if ((strcmp(MQTT_msg_in, "2") == 0))
                debug_Action("Sensor 2 Selected");
            else if ((strcmp(MQTT_msg_in, "3") == 0))
                debug_Action("Sensor 3 Selected");
            else
                debug_Trouble("Say What?");
        }
        else if (strcmp(MQTT_command, "/Screen") == 0) // Screen control
        {
            if (strcmp(MQTT_msg_in, "") == 0)
            {
#ifdef DEBUG4
                debug_Trouble("No actual message...");
#endif
                // Might be a good way to request status...
            }
            else if ((strcmp(MQTT_msg_in, "1") == 0) || (strcmp(MQTT_msg_in, "ON") == 0))
                SSD1306_on();
            else if ((strcmp(MQTT_msg_in, "0") == 0) || (strcmp(MQTT_msg_in, "OFF") == 0))
                SSD1306_off();
            else if ((strcmp(MQTT_msg_in, "+") == 0) || (strcmp(MQTT_msg_in, "UP") == 0))
                SSD1306_Bright();
            else if ((strcmp(MQTT_msg_in, "-") == 0) || (strcmp(MQTT_msg_in, "DOWN") == 0))
                SSD1306_Dim();
            else
                debug_Trouble("Say What?");
        }
        else
        {
#ifdef DEBUG4
            debug_Trouble("Dunno Whatcha want...");
#endif
        }
    }
    else
    {
#ifdef DEBUG4
        debug_Trouble("But, it's TOO Bloody Big!");
#endif
    }
}

void MQTT_reconnect()
{
    char debugTEXT[46];

    // Loop until we're reconnected
    while (!MQTT_client.connected())
    {
        debug_SectionTitle("Attempting MQTT connection...");
        // Create a random client ID
        String clientId = MQTT_ClientName;
        clientId += String(random(0xffff), HEX);
        // Attempt to connect
        if (MQTT_client.connect(clientId.c_str()))
        {
            SSD1306_Static(" MQTT good ", 3);
            delay(500);
            SSD1306_Static("           ", 3);
            sprintf(debugTEXT, "connected to %-24s", MQTT_broker);
            debug_LineOut(debugTEXT);
            sprintf(debugTEXT, "My Name:  %-27s", MQTT_ClientName);
            debug_LineOut(debugTEXT);
            // Once connected, publish an announcement...
            char MQTT_statTopic_Device[100];
            strcpy(MQTT_statTopic_Device, MQTT_statTopic);
            strcat(MQTT_statTopic_Device, "/HELLO");
            // MQTT_client.publish(MQTT_statTopic_Device, "world");
            MQTT_client.publish(MQTT_statTopic_Device, WiFi_ssid);
            // ... and resubscribe
            MQTT_client.subscribe(MQTT_inTopic);
        }
        else
        {

            if (WiFi.status() != WL_CONNECTED)
            {
                SSD1306_Static("  no WiFi  ", 3);
                debug_Trouble("UH OH!!!  No WiFi!!!");
            }
            else
            {
                SSD1306_Static("  no MQTT  ", 3);
            }

            sprintf(debugTEXT, "failed, rc=%d", MQTT_client.state());
            debug_Trouble(debugTEXT);
            debug_Trouble("trying again in 5 seconds");
            // Wait 5 seconds before retrying
            delay(5000);
        }
    }
}

void MQTT_beacon()
{
    /* Beacon signal published at set interval to indicate the device
   * is still powered up and actively connected to MQTT...
   * A keepalive of sorts
   * also updates state within MQTT so it can be captured for
   * indicator light elsewhere
   */
    char MQTT_teleTopic_Device[100];
    char WiFiSignal[5];
    strcpy(MQTT_teleTopic_Device, MQTT_teleTopic);
    strcat(MQTT_teleTopic_Device, "/beep");
    // if (getTimer(beacon_timer, BEACON_INTERVAL))
    // {
    sprintf(WiFiSignal, "%d dBm", WiFi_strength());

    // MQTT_client.publish(MQTT_teleTopic_Device, "boop");
    MQTT_client.publish(MQTT_teleTopic_Device, WiFiSignal);
    blinkLED(5);

#ifdef DEBUG4
    debug_Action("Beacon sent");
#endif
    // }
}

void MQTT_Status(char const *Device, char const *Status) // Send status messages
{
    char MQTT_statTopic_Device[100];
    strcpy(MQTT_statTopic_Device, MQTT_statTopic);
    strcat(MQTT_statTopic_Device, "/");
    strcat(MQTT_statTopic_Device, Device);

#ifdef DEBUG4
    char debugTEXT[46];

    sprintf(debugTEXT, "}- %16s = %-16s -{", Device, Status);
    debug_Trouble(debugTEXT);
#endif
    MQTT_client.publish(MQTT_statTopic_Device, Status);
}

void MQTT_SendJSON(char *JSONstream)
{
    char MQTT_teleTopic_Device[100];
    strcpy(MQTT_teleTopic_Device, MQTT_teleTopic);
    strcat(MQTT_teleTopic_Device, "/");
    strcat(MQTT_teleTopic_Device, "SENSOR");
    MQTT_client.publish(MQTT_teleTopic_Device, JSONstream);
}
