#include "libraries.h"
#include "functions.h"

////////////////////////////////////////////////////////////////
// These exist because the variables are declared elsewhere in
// the project.
extern char *WiFi_ssid;
extern char *WiFi_password;
extern char *WiFi_ClientName;
////////////////////////////////////////////////////////////////

// Initialize the WiFi
//
// If we don't get a connection, we just keep trying forever.
// Probably be an idea to eventually just give up, but most
// ESP8266 devices are kinda useless without the connection...
void WiFi_init()
{
    char debugTEXT[46];

    WiFi.mode(WIFI_STA); // Connecting as a STATION
    WiFi.begin(WiFi_ssid, WiFi_password);

    // We start by connecting to a WiFi network
    sprintf(debugTEXT, "Connecting to: %-22s", WiFi_ssid);
    debug_SectionTitle(debugTEXT);
    {
        debug_ProgressBar0();
        int dotcount = 0;
        while (WiFi.status() != WL_CONNECTED) // Give it a bit of time to establish the WiFi connection...
        {
            dotcount++;
            delay(500);
            debug_ProgressBar1();
            if (dotcount >= 46)
            {
                debug_ProgressBar2(dotcount);
                dotcount = 0;
                debug_ProgressBar0();
            }
            SSD1306_Static("  no WiFi  ", 3);
        }
        debug_ProgressBar2(dotcount);
    }
    sprintf(debugTEXT, "WiFi connected on Channel %-2d", WiFi.channel());
    debug_LineOut(debugTEXT);
    sprintf(debugTEXT, "IP address: %-15s", WiFi.localIP().toString().c_str());
    debug_LineOut(debugTEXT);

    sprintf(debugTEXT, "My name is NOT %-22s", wifi_station_get_hostname());
    debug_LineOut(debugTEXT);
    sprintf(debugTEXT, "Really... It's %-22s", WiFi_ClientName);
    debug_LineOut(debugTEXT);

    SSD1306_Static(" WiFi good ", 3);

    // Add "build_flags = -D DEBUG0" to your platformio.ini to get some extra WiFi stats
#ifdef DEBUG0
    WiFi.printDiag(Serial);
    sprintf(debugTEXT, "RSSI: %d dBm", WiFi.RSSI());
    debug_LineOut(debugTEXT);
    sprintf(debugTEXT, "HostName: %-13s !!!! BULLSHIT !!!!", WiFi.hostname().c_str());
    debug_LineOut(debugTEXT);
    // Apparently, the ESP8266WiFi library does not actually support DHCP completely
    // { https://www.reddit.com/r/arduino/comments/d6mvc7/getting_hostname_from_dhcp_with_esp8266/ }
    // { https://github.com/esp8266/Arduino/issues/5695 }
#endif
}

int WiFi_strength()
{
    return (WiFi.RSSI());
}