#include "libraries.h"
#include "functions.h"

#ifdef d_SDD1306
#include "StickDude.h"

#include "d-SSD1306.h"
#endif

extern bool WalkingDudeIndicator;
extern bool WavingDudeIndicator;

extern int DS18B20_Count;
extern int BME280_Count;
////////////////////////////////////////////////////////////////////////////////
bool SSD1306_Exists()
{
  Wire.beginTransmission(0x3C);

  // See if something acknowledged the transmission
  int response = Wire.endTransmission();
  if (response == 0)
  {
    return (true);
  }
  else if (response == 4) // unknown error
  {
    debug_Trouble("Unknown response at hexAddress 0x3C");
    return (false);
  }
  else
  {
    return (false);
  }
}
////////////////////////////////////////////////////////////////////////////////

// Initialize the display
void SSD1306_init()
{
#ifdef d_SDD1306

  {
    // SSD1306_SWITCHCAPVCC = generate display voltage from 3.3V internally
    // the 'true' means reset the display on begin.  Let's see if this stops the intermittant freeze...
    if (!display.begin(SSD1306_SWITCHCAPVCC, 0x3C, true))
    {
      debug_Trouble("SSD1306 allocation failed");
      debug_Trouble("++++++++++++++++ STOPPING!!! +++++++++++++++++");
      while (true) // Slam on da brakes!
        blinkLED(100);
      //  Does this ever actually happen?
      //  Or even NEED to???
    }
    else
    {
      display.clearDisplay();

      display.dim(true); // These displays are excessively bright

      // Set up the active (changing) parts of the display
      Dcanvas6.setFont(&FreeMonoBold9pt7b);

      // Set up the static (non-changing) parts of the display
      Scanvas3.setFont(&FreeMonoBold9pt7b);

      display.setTextColor(WHITE, BLACK);
      display.setTextSize(1);
      { ////////////////////////////////  Why do I have to check this here
        ////////////////////////////////  & can't avoid the setup if it's missing???
        if (SSD1306_Exists())
          debug_Success("SSD1306 initialized");
        else
          debug_Trouble("No SSD1306 found");
      }
    }
  }
#endif
}

// Place static text on the yellow part of the OLED display
// Accepts the string (max 9 characters)
void SSD1306_title(const char *Title)
{ // Max 9 characters...
#ifdef d_SDD1306
  Tcanvas0.setFont(&FreeMonoBold12pt7b);
  Tcanvas0.fillScreen(BLACK);
  Tcanvas0.setCursor(0, 14);
  Tcanvas0.print(Title);
  // Tcanvas0.drawBitmap(4, row_Y[Row - 1] - 9, Tcanvas0.getBuffer(), 120, 15, WHITE, BLACK);
  display.drawBitmap(4, 0, Tcanvas0.getBuffer(), 128, 15, WHITE, BLACK);
  display.display();
#endif
}

// Place static text on the blue part of the OLED display
// Accepts the string (max 11 characters) and the row number (1-3)
void SSD1306_Static(const char *Line, int Row)
{ // Max 11 characters
#ifdef d_SDD1306
  Scanvas3.fillScreen(BLACK);
  Scanvas3.setCursor(0, 11);
  Scanvas3.print(Line);
  if (strlen(Line) > 0)
    Scanvas3.print(":"); // Add a colon if there's actual content on the line...
  display.drawBitmap(4, row_Y[Row - 1], Scanvas3.getBuffer(), 120, 15, WHITE, BLACK);

  display.display();
#endif
}

// Display the temperatures on the OLED display
char TempText[10][6];
// DS18B20_Readings
void SSD1306_Display_DS18B20_Temps(struct DS18B20_Readings Data)
{
#ifdef d_SDD1306
  int Count;
  for (Count = 0; Count < DS18B20_Count; Count++)
    Data.Temperature[Count] = DS18B20_getReading(Count);

  for (int Count = 0; Count < DS18B20_Count; Count++)
  {
    if (Data.Temperature[Count] == -127)
    {
      sprintf(TempText[Count], "-----");
      Data.Temperature[Count] = 0;
    }
    else
      sprintf(TempText[Count], "%5.1f", Data.Temperature[Count]);
    SSD1306_Static(Data.Name[Count], Count + 1);
  }
  while (Count < 3) // Clear unused labels
    SSD1306_Static("", ++Count);

  SSD1306_title("DS18B20");

  Dcanvas6.fillScreen(BLACK);
  Dcanvas6.setCursor(0, 11);
  Dcanvas6.print(TempText[0]);
  display.drawBitmap(62, row_Y[0], Dcanvas6.getBuffer(), 65, 12, WHITE, BLACK);

  // display.display();

  Dcanvas6.fillScreen(BLACK);
  Dcanvas6.setCursor(0, 11);
  Dcanvas6.print(TempText[1]);
  display.drawBitmap(62, row_Y[1], Dcanvas6.getBuffer(), 65, 12, WHITE, BLACK);

  // display.display();

  Dcanvas6.fillScreen(BLACK);
  Dcanvas6.setCursor(0, 11);
  Dcanvas6.print(TempText[2]);
  display.drawBitmap(62, row_Y[2], Dcanvas6.getBuffer(), 65, 12, WHITE, BLACK);

  // display.display();
#endif
}

// BME280_Readings
void SSD1306_Display_BME280_Temps(struct BME280_Readings Datum)
{
#ifdef d_SDD1306
  SSD1306_title("BME280");

  SSD1306_Static("Temp:", 1);
  SSD1306_Static("Humi:", 2);
  SSD1306_Static("Baro:", 3);

  char TempText[6];
  char HumiText[6];
  char PresText[6];

  sprintf(TempText, "%6.1f", Datum.Temperature);
  Dcanvas6.fillScreen(BLACK);
  Dcanvas6.setCursor(0, 11);
  Dcanvas6.print(TempText);
  display.drawBitmap(62, row_Y[0], Dcanvas6.getBuffer(), 65, 12, WHITE, BLACK);

  sprintf(HumiText, "%6.1f", Datum.Humidity);
  Dcanvas6.fillScreen(BLACK);
  Dcanvas6.setCursor(0, 11);
  Dcanvas6.print(HumiText);
  display.drawBitmap(62, row_Y[1], Dcanvas6.getBuffer(), 65, 12, WHITE, BLACK);

  sprintf(PresText, "%06.1f", Datum.SeaPressure);
  Dcanvas6.fillScreen(BLACK);
  Dcanvas6.setCursor(0, 11);
  Dcanvas6.print(PresText);
  display.drawBitmap(62, row_Y[2], Dcanvas6.getBuffer(), 65, 12, WHITE, BLACK);
#endif
}

// Si7021_Readings
void SSD1306_Display_Si7021_Temps(struct Si7021_Readings Datum)
{
#ifdef d_SDD1306
  SSD1306_title("Si7021");

  SSD1306_Static("Temp:", 1);
  SSD1306_Static("Humi:", 2);
  SSD1306_Static("", 3);

  char TempText[6];
  char HumiText[6];

  sprintf(TempText, "%6.1f", Datum.Temperature);
  Dcanvas6.fillScreen(BLACK);
  Dcanvas6.setCursor(0, 11);
  Dcanvas6.print(TempText);
  display.drawBitmap(62, row_Y[0], Dcanvas6.getBuffer(), 65, 12, WHITE, BLACK);

  sprintf(HumiText, "%6.1f", Datum.Humidity);
  Dcanvas6.fillScreen(BLACK);
  Dcanvas6.setCursor(0, 11);
  Dcanvas6.print(HumiText);
  display.drawBitmap(62, row_Y[1], Dcanvas6.getBuffer(), 65, 12, WHITE, BLACK);

  Dcanvas6.fillScreen(BLACK);
  Dcanvas6.setCursor(0, 11);
  display.drawBitmap(62, row_Y[2], Dcanvas6.getBuffer(), 65, 12, WHITE, BLACK);
#endif
}

// HiH6130_Readings
void SSD1306_Display_HiH6130_Temps(struct hih6130_Readings Datum)
{
#ifdef d_SDD1306
  SSD1306_title("HiH6130");

  SSD1306_Static("Temp:", 1);
  SSD1306_Static("Humi:", 2);
  SSD1306_Static("", 3);

  char TempText[6];
  char HumiText[6];

  sprintf(TempText, "%6.1f", Datum.Temperature);
  Dcanvas6.fillScreen(BLACK);
  Dcanvas6.setCursor(0, 11);
  Dcanvas6.print(TempText);
  display.drawBitmap(62, row_Y[0], Dcanvas6.getBuffer(), 65, 12, WHITE, BLACK);

  sprintf(HumiText, "%6.1f", Datum.Humidity);
  Dcanvas6.fillScreen(BLACK);
  Dcanvas6.setCursor(0, 11);
  Dcanvas6.print(HumiText);
  display.drawBitmap(62, row_Y[1], Dcanvas6.getBuffer(), 65, 12, WHITE, BLACK);

  Dcanvas6.fillScreen(BLACK);
  Dcanvas6.setCursor(0, 11);
  display.drawBitmap(62, row_Y[2], Dcanvas6.getBuffer(), 65, 12, WHITE, BLACK);
#endif
}

// Create an activity indicator ////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
void SSD1306_Indicate_Action()
{
#ifdef d_SDD1306
  if (WalkingDudeIndicator)
    beerStickDude(DottyX, 64 - 12);

  if (WavingDudeIndicator)
    beerStickDude(120, 0);
#endif
}

// StickDude waving in the corner...
int DudeNumber = 0;
void WavingDude()
{
#ifdef d_SDD1306
  if (DudeNumber >= StickDude_wave_count)
    DudeNumber = 0;

  StickDudePIC.fillScreen(BLACK);
  StickDudePIC.drawBitmap(0, 0, StickDude_wave[DudeNumber], DUDE_WIDTH, DUDE_HEIGHT, 1);

  DudeNumber++;

  display.drawBitmap(120, 0, StickDudePIC.getBuffer(), 8, 12, WHITE, BLACK);
  display.display();
#endif
}

// Just a little dude that walks across the bottom edge of the display
int DudeStep = 0;
void WalkingDude()
{
#ifdef d_SDD1306
  DottyX++;
  if (DottyX > SCREEN_WIDTH)
    DottyX = 0;
  if (DudeStep >= StickDude_walk_count)
    DudeStep = 0;

  StickDudePIC.fillScreen(BLACK);
  StickDudePIC.drawBitmap(0, 0, StickDude_walk[DudeStep], DUDE_WIDTH, DUDE_HEIGHT, 1);

  DudeStep++;

  display.drawBitmap(DottyX, 64 - 12, StickDudePIC.getBuffer(), 8, 12, WHITE, BLACK);
  display.display();
#endif
}

// Dude stops for a beer
void beerStickDude(int LEFT, int TOP)
{
#ifdef d_SDD1306
  int DudeNum;

  for (DudeNum = 0; DudeNum < StickDude_beer_count; DudeNum++)
  {
    {
      StickDudePIC.fillScreen(BLACK);
      StickDudePIC.drawBitmap(0, 0, StickDude_beer[DudeNum], DUDE_WIDTH, DUDE_HEIGHT, 1);
      display.drawBitmap(LEFT, TOP, StickDudePIC.getBuffer(), 8, 12, WHITE, BLACK);
      display.display();
      delay(60);
    }
  }
  DudeNum--;
  for (; DudeNum >= 0; DudeNum--)
  {
    {
      StickDudePIC.fillScreen(BLACK);
      StickDudePIC.drawBitmap(0, 0, StickDude_beer[DudeNum], DUDE_WIDTH, DUDE_HEIGHT, 1);
      display.drawBitmap(LEFT, TOP, StickDudePIC.getBuffer(), 8, 12, WHITE, BLACK);
      display.display();
      delay(30);
    }
  }
#endif
}

/*
 * Possible future features
 */

// Turn display on/off ////////////////////////////////////////////////////////
void SSD1306_off()
{
  display.ssd1306_command(SSD1306_DISPLAYOFF);
  debug_Action("Screen OFF");
}

void SSD1306_on()
{
  display.ssd1306_command(SSD1306_DISPLAYON);
  debug_Action("Screen ON");
}

void SSD1306_Dim()
{
  display.dim(true); // These displays are excessively bright
  debug_Action("Screen DIM");
}

void SSD1306_Bright()
{
  display.dim(false); // These displays are excessively bright
  debug_Action("Screen BRIGHT");
}
