#include "libraries.h"
#include "functions.h"
#include "TopSecret.h"

// I2C pins suitable for the ESP-M3-Everything!
const int SSD1306_SDA = 14; // Tarduino D5
const int SSD1306_SCL = 12; // Tarduino D6

bool WalkingDudeIndicator = false;
bool WavingDudeIndicator = true;

SimpleTimer timer;

extern int DS18B20_Count;
extern int BME280_Count;
extern int Si7021_Count;
extern int hih6130_Count;

void setup()
{
  // Enable and turn off the On-board LED
  // (notice that it's inverted on most ESP boards)
  pinMode(LED_BUILTIN, OUTPUT);
  digitalWrite(LED_BUILTIN, HIGH);

  Serial.begin(115200);
  delay(500); // Delay to let the ESP get booted before sending out serial data
  debug_TitleScreen();

  if (I2C_Test(SSD1306_SDA, SSD1306_SCL)) // Force a reset of the I2C bus if needed
    Wire.begin(SSD1306_SDA, SSD1306_SCL); // Select SDA & SCL pins
  else
  {
    ////////////////////////////////  Need to send an MQTT error status message!
    debug_Action("POOP!");
  }
  debug_Separator();

  // SSD1306 ///////////////////////////////////////////////////////////////////
  SSD1306_init();

  // Display static text
  // SSD1306_title("ColdStuff");
  SSD1306_title(DISP_Title);
#ifdef DEBUG
  debug_LineOut("Title on-screen");
#endif
  debug_Separator();

  SSD1306_Static("  Booting  ", 3);

  debug_Action("Finding Sensors...");
// DS18B20 ///////////////////////////////////////////////////////////////////
#ifdef s_DS18B20
  DS18B20_Count = DS18B20_init();
#endif

  // BME280 ////////////////////////////////////////////////////////////////////
#ifdef s_BME280
  BME280_Count = BME280_init();
#endif

  // Si7021 ////////////////////////////////////////////////////////////////////
#ifdef s_Si7021
  Si7021_Count = Si7021_init();
#endif

  // Hih6130 ///////////////////////////////////////////////////////////////////
#ifdef s_Hih6130
  hih6130_Count = HiH6130_init();
#endif

  // Interval Timers ///////////////////////////////////////////////////////////
  debug_Separator();
  debug_SectionTitle("Interval Timers");

  timer.setInterval(20, DOTheMQTT);
  debug_LineOut("MQTT");

#ifdef s_DS18B20
  {
    timer.setInterval(10000, DS18B20_main);
    debug_LineOut("DS18B20 Handler");
  }
#endif

#ifdef s_BME280
  {
    timer.setInterval(10000, BME280_main);
    debug_LineOut("BME280 Handler");
  }
#endif

#ifdef s_Si7021
  {
    timer.setInterval(10000, Si7021_main);
    debug_LineOut("Si7021 Handler");
  }
#endif

#ifdef s_Hih6130
  {
    timer.setInterval(10000, Hih6130_main);
    debug_LineOut("Hih6130 Handler");
  }
#endif

  {
    // timer.setInterval(30000, MQTT_beacon);
  }

  if (WalkingDudeIndicator)
  {
    timer.setInterval(80, WalkingDude); // Walking Dude across the bottom row
    debug_LineOut("Walking Dude Indicator");
  }

  if (WavingDudeIndicator)
  {
    timer.setInterval(100, WavingDude); // Waving Dude in the top-right corner
    debug_LineOut("Waving Dude Indicator");
  }

  debug_Separator();

  // WiFi //////////////////////////////////////////////////////////////////////
  WiFi_init();
  debug_Separator();

  // MQTT //////////////////////////////////////////////////////////////////////
  MQTT_init();

  // READY /////////////////////////////////////////////////////////////////////
  debug_ReadyScreen();
}

void loop()
{
  // Run the interval timers
  // Running processes here in loop() is a synchronous flow.
  // Running processes via the interval timers is assynchronous.
  // If your actions don't depend directly on each other,
  // synchronous flow makes no sense at all...
  timer.run();
}
