#ifdef s_BME280

/*
 * Based on:
 * https://randomnerdtutorials.com/esp8266-bme280-arduino-ide/
 */

#include "libraries.h"
#include "functions.h"

#include <Adafruit_BME280.h>

#include "s-BME280.h"

extern int Altitude;

void BME280_main()
{
    struct BME280_Readings BBB;

    for (int Count = 0; Count < BME280_Count; Count++)
    {
        BBB = BME280_getReadings(0);
    }

    SSD1306_Display_BME280_Temps(BBB);

    SSD1306_Indicate_Action();

    BME280_SendData(BBB);

    BME280_debug(BBB);
}

int BME280_init()
{
    bool status;
    char debugTEXT[46];

    // default settings
    // (you can also pass in a Wire library object like &Wire2)
    status = bme.begin(0x76);
    if (!status)
    {
        debug_LineOut("No BME280(s)");
    }
    else
    {
        sprintf(debugTEXT, "%2d BME280(s) initialised", 1);
        debug_LineOut(debugTEXT);
    }

    // Currently only capable of using one...

    return (1); // This needs to change
}

struct BME280_Readings BME280_getReadings(int deviceNumber)
{
    struct BME280_Readings AAA;

    AAA.Temperature = bme.readTemperature();
    AAA.Humidity = bme.readHumidity();
    AAA.AbsPressure = bme.readPressure() / 100.0F;

    // Attempting to derive SeaPressure based on formula found at:
    // https://keisan.casio.com/exec/system/1224575267
    AAA.SeaPressure = AAA.AbsPressure * pow((1 - ((0.0065 * Altitude) / (AAA.Temperature + (0.0065 * Altitude) + 273.15))), -5.257);

    return (AAA);
}

void BME280_SendData(struct BME280_Readings Data)
{
    char BME280_JSON[100];

    sprintf(BME280_JSON,
            "{\"Type\":\"BME280\", \"Temperature\":%5.1f, \"Humidity\":%5.1f, \"AbsPressure\":%5.2f, \"SeaPressure\":%5.2f}",
            Data.Temperature,
            Data.Humidity,
            Data.AbsPressure,
            Data.SeaPressure);

    MQTT_SendJSON(BME280_JSON);
}

void BME280_debug(struct BME280_Readings Datum)
{
#ifdef DEBUG2
    char debugTEXT[46];

    debug_SectionTitle("BME280:");
    debug_LineOut(debugTEXT);
    sprintf(debugTEXT, "Temperature:    %5.1f C", Datum.Temperature);
    debug_LineOut(debugTEXT);
    sprintf(debugTEXT, "Temperature:    %5.1f F", 1.8 * Datum.Temperature + 32);
    debug_LineOut(debugTEXT);
    sprintf(debugTEXT, "Humidity:       %5.1f %%", Datum.Humidity);
    debug_LineOut(debugTEXT);
    sprintf(debugTEXT, "Pressure:      %5.2f hPa", Datum.AbsPressure);
    debug_LineOut(debugTEXT);
    sprintf(debugTEXT, "SeaPressure:  %5.2f hPa", Datum.SeaPressure);
    debug_LineOut(debugTEXT);
#endif
}

#endif // s_BME280