#ifdef s_DS18B20

#include "libraries.h"
#include "functions.h"

#include "s-DS18B20.h"

void DS18B20_main()
{
  DS18B20_Readings Data;

  for (int Count = 0; Count < DS18B20_Count; Count++)
  {
    Data.Name[Count] = DS18B20_Labels[Count];
    Data.Temperature[Count] = DS18B20_getReading(Count);
  }

#ifndef SugarBush
  SSD1306_Display_DS18B20_Temps(Data);
#endif

  DS18B20_SendData(Data);

  SSD1306_Indicate_Action();

  DS18B20_debug(Data);
}

// Start the DS18B20 sensor(s)
// returns number of devices found, if you're interested...
int DS18B20_init()
{
  sensors.begin();

  // Grab a count of devices on the wire
  int DS18B20_Count = sensors.getDeviceCount();
  if (DS18B20_Count > 0)
  {
    char debugTEXT[46];

    sprintf(debugTEXT, "%2d DS18B20(s) initialised", DS18B20_Count);
    debug_LineOut(debugTEXT);
  }
  return (DS18B20_Count);
}

// Read a DS18B20 sensor
// Accepts a device number (in case you have more than one)
// Returns the current reading (Celsius because Farhenheit is for losers)
// (If you're a loser...  Change getTempCByIndex to getTempFByIndex :)
float DS18B20_getReading(int deviceNumber)
{
  sensors.requestTemperatures();
  return (sensors.getTempCByIndex(deviceNumber));
}

void DS18B20_SendData(struct DS18B20_Readings Data)
{
  char Partial[20];

  char DS18B20_JSON[100];

  sprintf(DS18B20_JSON, "{\"Type\":\"DS18B20\"");

  for (int Count = 0; Count < DS18B20_Count; Count++)
  {

    if (strlen(Data.Name[Count]) > 0)
    {
      sprintf(Partial, ", \"%s\": %5.2f", Data.Name[Count], Data.Temperature[Count]);
      strcat(DS18B20_JSON, Partial);
    }
  }
  strcat(DS18B20_JSON, "}");

  MQTT_SendJSON(DS18B20_JSON);
}

void DS18B20_debug(struct DS18B20_Readings Data)
{
#ifdef DEBUG2
  char debugTEXT[46];

  debug_SectionTitle("DS18B20(s):");
  for (int Count = 0; Count < DS18B20_Count; Count++)
    sprintf(debugTEXT, "%2d: %-6s       %5.2f C",
            Count,
            Data.Name[Count],
            Data.Temperature[Count]);
  debug_LineOut(debugTEXT);
#endif
}

#endif // s_DS18B20