#ifdef s_Hih6130

#include "libraries.h"
#include "functions.h"

#include "s-Hih6130.h"

void Hih6130_main()
{
    struct hih6130_Readings BBB;

    for (int Count = 0; Count < hih6130_Count; Count++)
    {
        BBB = hih6130_getReadings(0);
    }

    SSD1306_Display_HiH6130_Temps(BBB);

    SSD1306_Indicate_Action();

    hih6130_SendData(BBB);

    hih6130_debug(BBB);
}

int HiH6130_init()
{
    char debugTEXT[46];
    // This is a bunch of lies...
    // There's no actual initialisation being done so far...
    sprintf(debugTEXT, "%2d HiH6130(s) initialised", 1);
    debug_LineOut(debugTEXT);
    return (1);
}

struct hih6130_Readings hih6130_getReadings(int deviceNumber)
{
    struct hih6130_Readings AAA;

    byte Hum_H, Hum_L, Temp_H, Temp_L;
    unsigned int H_dat, T_dat;

    Wire.beginTransmission(HIHaddress);
    Wire.endTransmission();
    delay(100);

    Wire.requestFrom((int)HIHaddress, (int)4);
    Hum_H = Wire.read();
    Hum_L = Wire.read();
    Temp_H = Wire.read();
    Temp_L = Wire.read();
    Wire.endTransmission();

    AAA.Status = (Hum_H >> 6) & 0x03;

    Hum_H = Hum_H & 0x3f;
    H_dat = (((unsigned int)Hum_H) << 8) | Hum_L;

    T_dat = (((unsigned int)Temp_H) << 8) | Temp_L;
    T_dat = T_dat / 4;

    AAA.Temperature = T_dat * 1.007e-2 - 40.0;
    AAA.Humidity = H_dat * 6.10e-3;

    return (AAA);
}

void hih6130_SendData(struct hih6130_Readings Data)
{
    // hih6130_test();
    char hih6130_JSON[100];

    sprintf(hih6130_JSON,
            "{\"Type\":\"HiH6130\", \"Temperature\":%5.1f, \"Humidity\":%5.1f}",
            Data.Temperature,
            Data.Humidity);

    MQTT_SendJSON(hih6130_JSON);
}

void hih6130_debug(struct hih6130_Readings Datum)
{
#ifdef DEBUG2
    char debugTEXT[46];

    debug_SectionTitle("HiH6130:");
    sprintf(debugTEXT, "Temperature:   % 3.2f C", Datum.Temperature);
    debug_LineOut(debugTEXT);
    // sprintf(debugTEXT, "Temperature:   % 3.2f F", Datum.Temperature_F);
    // debug_LineOut(debugTEXT);
    sprintf(debugTEXT, "Humidity:      % 3.2f %%", Datum.Humidity);
    debug_LineOut(debugTEXT);
    // sprintf(debugTEXT, "Heat Index:  % 3.2f C", Datum.HeatIndex);
    // debug_LineOut(debugTEXT);
    // sprintf(debugTEXT, "Heat Index:  % 3.2f F", Datum.HeatIndex_F);
    // debug_LineOut(debugTEXT);
#endif
}

#endif // s_Hih6130