#ifdef s_Si7021

#include "libraries.h"
#include "functions.h"

#include <Adafruit_Si7021.h>

#include "s-Si7021.h"

void Si7021_main()
{
    struct Si7021_Readings BBB;

    for (int Count = 0; Count < Si7021_Count; Count++)
    {
        BBB = Si7021_getReadings(0);
    }

    SSD1306_Display_Si7021_Temps(BBB);

    SSD1306_Indicate_Action();

    Si7021_SendData(BBB);

    Si7021_debug(BBB);
}

int Si7021_init()
{
    bool status;
    char debugTEXT[46];

    status = Si7021.begin();
    if (!status)
    {
        debug_LineOut("No Si7021(s)");
    }
    else
    {
        sprintf(debugTEXT, "%2d Si7021(s) initialised", 1);
        debug_LineOut(debugTEXT);

#ifdef DEBUG3
        const char *ModelName[5] = {
            "SI eng. sample",
            "Si7013",
            "Si7020",
            "Si7021",
            "Unknown"};

        sprintf(debugTEXT, "   %-14s (Rev %d)", ModelName[Si7021.getModel()], Si7021.getRevision());
        debug_LineOut(debugTEXT);
        sprintf(debugTEXT, "   Serial #%X%X", Si7021.sernum_a, Si7021.sernum_b);
        debug_LineOut(debugTEXT);
#endif
    }

    // Currently only capable of using one...

    return (1); // This needs to change
}

struct Si7021_Readings Si7021_getReadings(int deviceNumber)
{
    struct Si7021_Readings AAA;

    AAA.Temperature = Si7021.readTemperature();
    AAA.Humidity = Si7021.readHumidity();

    return (AAA);
}

void Si7021_SendData(struct Si7021_Readings Data)
{
    char Si7021_JSON[100];

    sprintf(Si7021_JSON,
            "{\"Type\":\"Si7021\", \"Temperature\":%5.1f, \"Humidity\":%5.1f}",
            Data.Temperature,
            Data.Humidity);

    MQTT_SendJSON(Si7021_JSON);
}

void Si7021_debug(struct Si7021_Readings Datum)
{
#ifdef DEBUG2
    char debugTEXT[46];

    debug_SectionTitle("Si7021:");
    sprintf(debugTEXT, "Temperature:    %5.1f C", Datum.Temperature);
    debug_LineOut(debugTEXT);
    // sprintf(debugTEXT, "Temperature = %5.1f *F", 1.8 * Datum.Temperature + 32);
    // debug_LineOut(debugTEXT);
    sprintf(debugTEXT, "Humidity:       %5.1f %%", Datum.Humidity);
    debug_LineOut(debugTEXT);
#endif
}

#endif // s_Si7021